from machine import Pin
from machine import SPI
import time

# *********************************************
# Hardware GPIO Pin Configuration
# *********************************************
ENC28J60_SPI_SO              = 16
ENC28J60_SPI_SI              = 19
ENC28J60_SPI_SCK             = 18
ENC28J60_SPI_CS              = 17
ENC28J60_SPI_INT             = 21
ENC28J60_SPI_RESET           = 22
# *********************************************

ADDR_MASK                    = 0x1F
BANK_MASK                    = 0x60
SPRD_MASK                    = 0x80
                             # All-bank registers
EIE                          = 0x1B
EIR                          = 0x1C
ESTAT                        = 0x1D
ECON2                        = 0x1E
ECON1                        = 0x1F
                             # Bank 0 registers
ERDPTL                       = (0x00|0x00)
ERDPTH                       = (0x01|0x00)
EWRPTL                       = (0x02|0x00)
EWRPTH                       = (0x03|0x00)
ETXSTL                       = (0x04|0x00)
ETXSTH                       = (0x05|0x00)
ETXNDL                       = (0x06|0x00)
ETXNDH                       = (0x07|0x00)
ERXSTL                       = (0x08|0x00)
ERXSTH                       = (0x09|0x00)
ERXNDL                       = (0x0A|0x00)
ERXNDH                       = (0x0B|0x00)
ERXRDPTL                     = (0x0C|0x00)
ERXRDPTH                     = (0x0D|0x00)
ERXWRPTL                     = (0x0E|0x00)
ERXWRPTH                     = (0x0F|0x00)
EDMASTL                      = (0x10|0x00)
EDMASTH                      = (0x11|0x00)
EDMANDL                      = (0x12|0x00)
EDMANDH                      = (0x13|0x00)
EDMADSTL                     = (0x14|0x00)
EDMADSTH                     = (0x15|0x00)
EDMACSL                      = (0x16|0x00)
EDMACSH                      = (0x17|0x00)
                             # Bank 1 registers
EHT0                         = (0x00|0x20)
EHT1                         = (0x01|0x20)
EHT2                         = (0x02|0x20)
EHT3                         = (0x03|0x20)
EHT4                         = (0x04|0x20)
EHT5                         = (0x05|0x20)
EHT6                         = (0x06|0x20)
EHT7                         = (0x07|0x20)
EPMM0                        = (0x08|0x20)
EPMM1                        = (0x09|0x20)
EPMM2                        = (0x0A|0x20)
EPMM3                        = (0x0B|0x20)
EPMM4                        = (0x0C|0x20)
EPMM5                        = (0x0D|0x20)
EPMM6                        = (0x0E|0x20)
EPMM7                        = (0x0F|0x20)
EPMCSL                       = (0x10|0x20)
EPMCSH                       = (0x11|0x20)
EPMOL                        = (0x14|0x20)
EPMOH                        = (0x15|0x20)
EWOLIE                       = (0x16|0x20)
EWOLIR                       = (0x17|0x20)
ERXFCON                      = (0x18|0x20)
EPKTCNT                      = (0x19|0x20)
                             # Bank 2 registers
MACON1                       = (0x00|0x40|0x80)
MACON2                       = (0x01|0x40|0x80)
MACON3                       = (0x02|0x40|0x80)
MACON4                       = (0x03|0x40|0x80)
MABBIPG                      = (0x04|0x40|0x80)
MAIPGL                       = (0x06|0x40|0x80)
MAIPGH                       = (0x07|0x40|0x80)
MACLCON1                     = (0x08|0x40|0x80)
MACLCON2                     = (0x09|0x40|0x80)
MAMXFLL                      = (0x0A|0x40|0x80)
MAMXFLH                      = (0x0B|0x40|0x80)
MAPHSUP                      = (0x0D|0x40|0x80)
MICON                        = (0x11|0x40|0x80)
MICMD                        = (0x12|0x40|0x80)
MIREGADR                     = (0x14|0x40|0x80)
MIWRL                        = (0x16|0x40|0x80)
MIWRH                        = (0x17|0x40|0x80)
MIRDL                        = (0x18|0x40|0x80)
MIRDH                        = (0x19|0x40|0x80)
                             # Bank 3 registers
MAADR1                       = (0x00|0x60|0x80)
MAADR0                       = (0x01|0x60|0x80)
MAADR3                       = (0x02|0x60|0x80)
MAADR2                       = (0x03|0x60|0x80)
MAADR5                       = (0x04|0x60|0x80)
MAADR4                       = (0x05|0x60|0x80)
EBSTSD                       = (0x06|0x60)
EBSTCON                      = (0x07|0x60)
EBSTCSL                      = (0x08|0x60)
EBSTCSH                      = (0x09|0x60)
MISTAT                       = (0x0A|0x60|0x80)
EREVID                       = (0x12|0x60)
ECOCON                       = (0x15|0x60)
EFLOCON                      = (0x17|0x60)
EPAUSL                       = (0x18|0x60)
EPAUSH                       = (0x19|0x60)
                             # PHY registers
PHCON1                       = 0x00
PHSTAT1                      = 0x01
PHHID1                       = 0x02
PHHID2                       = 0x03
PHCON2                       = 0x10
PHSTAT2                      = 0x11
PHIE                         = 0x12
PHIR                         = 0x13
PHLCON                       = 0x14
                             # ENC28J60 ERXFCON Register Bit Definitions
ERXFCON_UCEN                 = 0x80
ERXFCON_ANDOR                = 0x40
ERXFCON_CRCEN                = 0x20
ERXFCON_PMEN                 = 0x10
ERXFCON_MPEN                 = 0x08
ERXFCON_HTEN                 = 0x04
ERXFCON_MCEN                 = 0x02
ERXFCON_BCEN                 = 0x01
                             # ENC28J60 EIE Register Bit Definitions
EIE_INTIE                    = 0x80
EIE_PKTIE                    = 0x40
EIE_DMAIE                    = 0x20
EIE_LINKIE                   = 0x10
EIE_TXIE                     = 0x08
EIE_WOLIE                    = 0x04
EIE_TXERIE                   = 0x02
EIE_RXERIE                   = 0x01
                             # ENC28J60 EIR Register Bit Definitions
EIR_PKTIF                    = 0x40
EIR_DMAIF                    = 0x20
EIR_LINKIF                   = 0x10
EIR_TXIF                     = 0x08
EIR_WOLIF                    = 0x04
EIR_TXERIF                   = 0x02
EIR_RXERIF                   = 0x01
                             # ENC28J60 ESTAT Register Bit Definitions
ESTAT_INT                    = 0x80
ESTAT_LATECOL                = 0x10
ESTAT_RXBUSY                 = 0x04
ESTAT_TXABRT                 = 0x02
ESTAT_CLKRDY                 = 0x01
                             # ENC28J60 ECON2 Register Bit Definitions
ECON2_AUTOINC                = 0x80
ECON2_PKTDEC                 = 0x40
ECON2_PWRSV                  = 0x20
ECON2_VRPS                   = 0x08
                             # ENC28J60 ECON1 Register Bit Definitions
ECON1_TXRST                  = 0x80
ECON1_RXRST                  = 0x40
ECON1_DMAST                  = 0x20
ECON1_CSUMEN                 = 0x10
ECON1_TXRTS                  = 0x08
ECON1_RXEN                   = 0x04
ECON1_BSEL1                  = 0x02
ECON1_BSEL0                  = 0x01
                             # ENC28J60 MACON1 Register Bit Definitions
MACON1_LOOPBK                = 0x10
MACON1_TXPAUS                = 0x08
MACON1_RXPAUS                = 0x04
MACON1_PASSALL               = 0x02
MACON1_MARXEN                = 0x01
                             # ENC28J60 MACON2 Register Bit Definitions
MACON2_MARST                 = 0x80
MACON2_RNDRST                = 0x40
MACON2_MARXRST               = 0x08
MACON2_RFUNRST               = 0x04
MACON2_MATXRST               = 0x02
MACON2_TFUNRST               = 0x01
                             # ENC28J60 MACON3 Register Bit Definitions
MACON3_PADCFG2               = 0x80
MACON3_PADCFG1               = 0x40
MACON3_PADCFG0               = 0x20
MACON3_TXCRCEN               = 0x10
MACON3_PHDRLEN               = 0x08
MACON3_HFRMLEN               = 0x04
MACON3_FRMLNEN               = 0x02
MACON3_FULDPX                = 0x01
                             # ENC28J60 MICMD Register Bit Definitions
MICMD_MIISCAN                = 0x02
MICMD_MIIRD                  = 0x01
                             # ENC28J60 MISTAT Register Bit Definitions
MISTAT_NVALID                = 0x04
MISTAT_SCAN                  = 0x02
MISTAT_BUSY                  = 0x01
                             # ENC28J60 PHY PHCON1 Register Bit Definitions
PHCON1_PRST                  = 0x8000
PHCON1_PLOOPBK               = 0x4000
PHCON1_PPWRSV                = 0x0800
PHCON1_PDPXMD                = 0x0100
                             # ENC28J60 PHY PHSTAT1 Register Bit Definitions
PHSTAT1_PFDPX                = 0x1000
PHSTAT1_PHDPX                = 0x0800
PHSTAT1_LLSTAT               = 0x0004
PHSTAT1_JBSTAT               = 0x0002
                             # ENC28J60 PHY PHCON2 Register Bit Definitions
PHCON2_FRCLINK               = 0x4000
PHCON2_TXDIS                 = 0x2000
PHCON2_JABBER                = 0x0400
PHCON2_HDLDIS                = 0x0100
                             # ENC28J60 Packet Control Byte Bit Definitions
PKTCTRL_PHUGEEN              = 0x08
PKTCTRL_PPADEN               = 0x04
PKTCTRL_PCRCEN               = 0x02
PKTCTRL_POVERRIDE            = 0x01
                             # SPI operation codes
ENC28J60_READ_CTRL_REG       = 0x00
ENC28J60_READ_BUF_MEM        = 0x3A
ENC28J60_WRITE_CTRL_REG      = 0x40
ENC28J60_WRITE_BUF_MEM       = 0x7A
ENC28J60_BIT_FIELD_SET       = 0x80
ENC28J60_BIT_FIELD_CLR       = 0xA0
ENC28J60_SOFT_RESET          = 0xFF
                                     # The RXSTART_INIT should be zero. See Rev. B4 Silicon Errata
                                     # buffer boundaries applied to internal 8K ram
                                     # the entire available packet buffer space is allocated
RXSTART_INIT     = 0x00              # start with recbuf at 0/
RXSTOP_INIT      = (0x1FFF-0x0600-1) # receive buffer end
TXSTART_INIT     = (0x1FFF-0x0600)   # start TX buffer at 0x1FFF-0x0600, pace for one full ethernet frame (~1500 bytes)
TXSTOP_INIT      = 0x1FFF      # stp TX buffer at end of mem
MAX_FRAMELEN     = 1500        # max frame length which the conroller will accept:
                               # (note: maximum ethernet frame length would be 1518)

def gen_Macaddress():
    '''
    a[0] bit 2-7, a[1],a[2],a[3],a[4],a[5] = MAC address

                                 Network cable Output sequence
    Mac Address <47:40> = a[0]   a[0] bit 40 -> 41 -> 42 -> 43 -> 44 -> 45 -> 46 -> 47
                                 a[0] bit 40 = 0: Unicast address 1: Multicast address
                                 a[0] bit 41 = 0: Global address  1: Local address
    Mac Address <39:32> = a[1]   a[1] bit 32 -> 33 -> 34 -> 35 -> 36 -> 37 -> 38 -> 39
    Mac Address <31:24> = a[2]   a[2] bit 24 -> 25 -> 26 -> 27 -> 28 -> 29 -> 30 -> 31
    Mac Address <23:16> = a[3]   a[3] bit 16 -> 17 -> 18 -> 19 -> 20 -> 21 -> 22 -> 23
    Mac Address <15:8>  = a[4]   a[4] bit 8 -> 9 -> 10 -> 11 -> 12 -> 13 -> 14 -> 15
    Mac Address <7:0>   = a[5]   a[5] bit 0 -> 1 -> 2 -> 3 -> 4 -> 5 -> 6 -> 7
    '''
    from machine import unique_id
    a = unique_id()[-6:]
    return bytes([(a[0] & 0xfe)| 0x02]) + a[1:] # set local assignment bit (IEEE802)
                                                # unicast mac address

class ENC28J60:
    def __init__(self):
        self.Enc28j60Bank = 0xFFFF
        self.flagInt = False
        self.int28j60 = False
        self.aintIntPin = ENC28J60_SPI_INT
        self.aintResetPin = ENC28J60_SPI_RESET
        # SPI init
        self.spi = SPI(0, baudrate=12000000, polarity=0, phase=0, sck=Pin(ENC28J60_SPI_SCK), mosi=Pin(ENC28J60_SPI_SI), miso=Pin(ENC28J60_SPI_SO))
        self.spiCS = Pin(ENC28J60_SPI_CS, Pin.OUT, value=1)

        # Interrupt Pin
        self.pInt = Pin(self.aintIntPin, Pin.IN,Pin.PULL_UP)
        # Rest Pin
        self.pReset = Pin(self.aintResetPin, Pin.OUT, value=1)

    def intCallback(cls,interruptPin):
        #print("Interrupt")
        #breakpoint()      # For debug in Callback
        cls.int28j60 = True

    def enableInt(self,callback):
        if self.flagInt == False:
            self.pInt.irq(trigger=Pin.IRQ_FALLING, handler=callback)
            self.flagInt = True

    def disableInt(self):
        if self.flagInt == True:
            self.pInt.init()
            self.flagInt = False

    def Enc28j60ReadOp(self, op, address):
        self.spiCS(0)
        rd = 0
        if (address & 0x80) == 0:
            bary = bytearray(2)
            bary[0] = op | (address & ADDR_MASK)
            self.spi.write_readinto(bary, bary)
            rd = bary[1]
        else:
            bary = bytearray(3)
            bary[0] = op | (address & ADDR_MASK)
            self.spi.write_readinto(bary, bary)
            rd = bary[2]
        self.spiCS(1)
        return rd

    def Enc28j60WriteOp(self, op, address, data):
        bary = bytearray(2)
        bary[0] = op | (address & ADDR_MASK)
        bary[1] = data & 0xFF
        self.spiCS(0)
        self.spi.write(bary)
        self.spiCS(1)

    def Enc28j60ReadBuffer(self , len, data):
        # issue read command
        buf = bytearray(1)
        self.spiCS(0)
        buf[0] = ENC28J60_READ_BUF_MEM
        self.spi.write(buf)
        for i in range(len):
            self.spi.readinto(buf)
            data[i] = buf[0]
        self.spiCS(1)

    def Enc28j60WriteBuffer(self, len, data):
        # issue write command
        cmd = bytearray([ENC28J60_WRITE_BUF_MEM, 0])
        self.spiCS(0)
        self.spi.write(cmd)
        self.spi.write(data[:len])
        self.spiCS(1)

    def Enc28j60SetBank(self, address):
        # set the bank (if needed)
        if (address & BANK_MASK) != self.Enc28j60Bank:
            # set the bank
            self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0))
            self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5)
            self.Enc28j60Bank = (address & BANK_MASK)

    def Enc28j60Read(self, address):
        # set the bank
        self.Enc28j60SetBank(address)
        # do the read
        return self.Enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address)

    def Enc28j60Write(self, address, data):
        # set the bank
        self.Enc28j60SetBank(address)
        # do the write
        self.Enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data)

    def Enc28j60PhyWrite(self, address, data):
        # set the PHY register address
        self.Enc28j60Write(MIREGADR, address)
        # write the PHY data
        self.Enc28j60Write(MIWRL, data & 0xFF)
        self.Enc28j60Write(MIWRH, data>>8);
        # wait until the PHY write completes
        while self.Enc28j60Read(MISTAT) & MISTAT_BUSY:
            time.sleep(0.0001)

    def Enc28j60PhyRead(self, address):
        while 0 != (self.Enc28j60Read(MISTAT) & MISTAT_BUSY):
            pass
        # set the PHY register address
        self.Enc28j60Write(MIREGADR, address);

        # Start read operation
        self.Enc28j60Write(MICMD, 0x01)

        # Wait for the read operation to complete
        while 0 != (self.Enc28j60Read(MISTAT) & MISTAT_BUSY):
            pass

        # Clear command register
        self.Enc28j60Write(MICMD, 0)

        # Read the lower 8 bits
        data = self.Enc28j60Read(MIRDL)
        # Read the upper 8 bits
        data |= self.Enc28j60Read(MIRDH) << 8

        return data

    def InitPhy(self):
        '''
          Flash the 2 RJ45 LEDs twice to show that the interface works
          Magjack leds configuration, see enc28j60 datasheet, page 11
          LEDA=green LEDB=yellow
        '''
        # 0x880 is PHLCON LEDB=on, LEDA=on
        # enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00)
        self.Enc28j60PhyWrite(PHLCON,0x880)
        time.sleep(0.5)
        #
        # 0x990 is PHLCON LEDB=off, LEDA=off
        # enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00)
        self.Enc28j60PhyWrite(PHLCON,0x990)
        time.sleep(0.5)
        #
        # 0x880 is PHLCON LEDB=on, LEDA=on
        # enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00)
        self.Enc28j60PhyWrite(PHLCON,0x880)
        time.sleep(0.5)
        #
        # 0x990 is PHLCON LEDB=off, LEDA=off
        # enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00)
        self.Enc28j60PhyWrite(PHLCON,0x990)
        time.sleep(0.5)
        # 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
        # enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10)
        self.Enc28j60PhyWrite(PHLCON,0x476)
        time.sleep(0.5)

    def Enc28j60deInit(self):
        self.spi.deinit()
        self.spiCS.init()
        self.pInt.init()
        self.pReset.init()

    def Enc28j60Init(self, macaddr):
        # MAC address
        self.myMacAddr = macaddr
        # perform system reset
        self.pReset.low()
        time.sleep(0.02)
        self.pReset.high()
        time.sleep(1)
        self.Enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET)
        time.sleep(0.2)
        self.pInt.irq(trigger=Pin.IRQ_FALLING, handler=self.intCallback)
        self.flagInt = True

        # check CLKRDY bit to see if reset is complete
        # The CLKRDY does not work. See Rev. B4 Silicon Errata point. Just wait.
        # while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY));
        # do bank 0 stuff
        # initialize receive buffer
        # 16-bit transfers, must write low byte first
        # set receive buffer start address
        self.NextPacketPtr = RXSTART_INIT
        #  Rx start
        self.Enc28j60Write(ERXSTL, RXSTART_INIT & 0xFF)
        self.Enc28j60Write(ERXSTH, RXSTART_INIT >> 8)
        # set receive pointer address
        self.Enc28j60Write(ERXRDPTL, RXSTART_INIT & 0xFF)
        self.Enc28j60Write(ERXRDPTH, RXSTART_INIT >> 8)
        # RX end
        self.Enc28j60Write(ERXNDL, RXSTOP_INIT & 0xFF)
        self.Enc28j60Write(ERXNDH, RXSTOP_INIT >> 8)
        # TX start
        self.Enc28j60Write(ETXSTL, TXSTART_INIT & 0xFF)
        self.Enc28j60Write(ETXSTH, TXSTART_INIT >> 8)
        # TX end
        self.Enc28j60Write(ETXNDL, TXSTOP_INIT & 0xFF)
        self.Enc28j60Write(ETXNDH, TXSTOP_INIT >> 8)
        # do bank 1 stuff, packet filter:
        # For broadcast packets we allow only ARP packtets
        # All other packets should be unicast only for our mac (MAADR)
        #
        # The pattern to match on is therefore
        # Type     ETH.DST
        # ARP      BROADCAST
        # 06 08 -- ff ff ff ff ff ff -> ip checksum for theses bytes=f7f9
        # in binary these poitions are:11 0000 0011 1111
        # This is hex 303F->EPMM0=0x3f,EPMM1=0x30
        self.Enc28j60Write(ERXFCON, ERXFCON_UCEN | ERXFCON_CRCEN | ERXFCON_PMEN)
        self.Enc28j60Write(EPMM0, 0x3f)
        self.Enc28j60Write(EPMM1, 0x30)
        self.Enc28j60Write(EPMCSL, 0xf9)
        self.Enc28j60Write(EPMCSH, 0xf7)
        #
        # do bank 2 stuff
        # enable MAC receive
        self.Enc28j60Write(MACON1, MACON1_MARXEN | MACON1_TXPAUS | MACON1_RXPAUS)
        # bring MAC out of reset
        self.Enc28j60Write(MACON2, 0x00)
        # enable automatic padding to 60bytes and CRC operations
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0 | MACON3_TXCRCEN | MACON3_FRMLNEN)
        # set inter-frame gap (non-back-to-back)
        self.Enc28j60Write(MAIPGL, 0x12)
        self.Enc28j60Write(MAIPGH, 0x0C)
        # set inter-frame gap (back-to-back)
        self.Enc28j60Write(MABBIPG, 0x12)
        # Set the maximum packet size which the controller will accept
        # Do not send packets longer than MAX_FRAMELEN:
        self.Enc28j60Write(MAMXFLL, MAX_FRAMELEN & 0xFF)	
        self.Enc28j60Write(MAMXFLH, MAX_FRAMELEN >> 8)
        # do bank 3 stuff
        # write MAC address
        # NOTE: MAC address in ENC28J60 is byte-backward
        self.Enc28j60Write(MAADR5, macaddr[0])
        self.Enc28j60Write(MAADR4, macaddr[1])
        self.Enc28j60Write(MAADR3, macaddr[2])
        self.Enc28j60Write(MAADR2, macaddr[3])
        self.Enc28j60Write(MAADR1, macaddr[4])
        self.Enc28j60Write(MAADR0, macaddr[5])
        # no loopback of transmitted frames
        self.Enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS)
        self.Enc28j60PhyRead(PHIR)  # clear PHY int flags
        self.Enc28j60PhyWrite(PHIE, 0x0002)   # set PHIE_PGEIE 
        # switch to bank 0
        self.Enc28j60SetBank(ECON1)
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, EIR, EIR_TXIF | EIR_TXERIF | EIR_RXERIF )
        # enable interrutps
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE | EIE_PKTIE)
        # enable packet reception
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN)
        self.enc28j60_dump_regs('Init() Exit')

    def Enc28j60getrev(self):
        # read the revision of the chip
        return self.Enc28j60Read(EREVID)

    def Enc28j60PacketSend(self, len, packet):
        # Set the write pointer to start of transmit buffer area
        self.Enc28j60Write(EWRPTL, TXSTART_INIT & 0xFF)
        self.Enc28j60Write(EWRPTH, TXSTART_INIT >> 8)
        # Set the TXND pointer to correspond to the packet size given
        self.Enc28j60Write(ETXNDL, (TXSTART_INIT + len) & 0xFF)
        self.Enc28j60Write(ETXNDH, (TXSTART_INIT + len) >> 8)
        # write per-packet control byte (0x00 means use macon3 settings)
        # copy the packet into the transmit buffer
        self.Enc28j60WriteBuffer(len, packet) # include write per-packet control byte
        # send the contents of the transmit buffer onto the network
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS)
        # Reset the transmit logic problem. See Rev. B4 Silicon Errata point 12.
        if (self.Enc28j60Read(EIR) & EIR_TXERIF):
            self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRTS)

    def Enc28j60PacketReceive(self, maxlen, packet):
        '''
        Gets a packet from the network receive buffer, if one is available.
        The packet will be headed by an ethernet header.
            maxlen  The maximum acceptable length of a retrieved packet.
            packet  Pointer where packet data should be stored.
        Returns: Packet length in bytes if a packet was retrieved, zero otherwise.
        '''
        # check if a packet has been received and buffered
        #if( !(enc28j60Read(EIR) & EIR_PKTIF) ){
        # The above does not work. See Rev. B4 Silicon Errata point 6.
        if self.Enc28j60Read(EPKTCNT) == 0:
            return 0

        # Set the read pointer to the start of the received packet
        self.Enc28j60Write(ERDPTL, self.NextPacketPtr & 0xff)
        self.Enc28j60Write(ERDPTH, self.NextPacketPtr >> 8)

        # 6-byte header
        r6b = bytearray(6)
        self.spiCS(0)
        self.spi.write(bytes([ENC28J60_READ_BUF_MEM]))
        self.spi.readinto(r6b)
        self.spiCS(1)
        # The first two bytes are the address of the next packet
        self.NextPacketPtr = r6b[0] | (r6b[1]<<8)

        # Get the length of the received packet
        # read the packet length (see datasheet page 43)
        len = r6b[2] | (r6b[3]<<8)
        len -= 4;  # remove the CRC count
        # limit retrieve length
        if len > (maxlen-1):
            len = maxlen - 1

        # Get the receive status vector (RSV)
        rxstat = r6b[4] | (r6b[5]<<8)

        # check CRC and symbol errors (see datasheet page 44, table 7-3):
        # The ERXFCON.CRCEN is set by default. Normally we should not
        # need to check this.
        if (rxstat & 0x80) == 0 or len <= 0:
            # invalid
            len = 0
        else:
            # copy the packet from the receive buffer
            self.Enc28j60ReadBuffer(len, packet);
        # Move the RX read pointer to the start of the next received packet
        # This frees the memory we just read out
        self.Enc28j60Write(ERXRDPTL, self.NextPacketPtr);
        self.Enc28j60Write(ERXRDPTH, self.NextPacketPtr >> 8);

        # decrement the packet counter indicate we are done with this packet
        self.Enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);
        return len

    def Enc28j60IsLinkUp(self):
        return 0 != (self.Enc28j60PhyRead(PHSTAT2) & 0x0400)

    def enc28j60_dump_regs(self, msg):
        print("..... {:s} .....".format(msg))
        print(". HwRevID: 0x{:02X}".format(self.Enc28j60Read(EREVID)))
        print(". Cntrl: ECON1 ECON2 ESTAT  EIR  EIE\n.        0x{:02X}  0x{:02X}  0x{:02X}  0x{:02X}  0x{:02X}".format(self.Enc28j60Read(ECON1), self.Enc28j60Read(ECON2), self.Enc28j60Read(ESTAT),self.Enc28j60Read(EIR), self.Enc28j60Read(EIE)))
        print(". MAC  : MACON1 MACON3 MACON4\n.        0x{:02X}   0x{:02X}   0x{:02X}".format(self.Enc28j60Read(MACON1),self.Enc28j60Read(MACON3), self.Enc28j60Read(MACON4)))
        print(". Rx   : ERXST  ERXND  ERXWRPT ERXRDPT ERXFCON EPKTCNT MAMXFL\n.        0x{:02X}{:02X} 0x{:02X}{:02X} 0x{:02X}{:02X}  0x{:02X}{:02X}  0x{:02X}    0x{:02X}    0x{:02X}{:02X}".format(self.Enc28j60Read(ERXSTH),self.Enc28j60Read(ERXSTL),self.Enc28j60Read(ERXNDH),self.Enc28j60Read(ERXNDL), self.Enc28j60Read(ERXWRPTH),self.Enc28j60Read(ERXWRPTL),self.Enc28j60Read(ERXRDPTH),self.Enc28j60Read(ERXRDPTL), self.Enc28j60Read(ERXFCON), self.Enc28j60Read(EPKTCNT),self.Enc28j60Read(MAMXFLH),self.Enc28j60Read(MAMXFLL)))
        print(". Tx   : ETXST  ETXND  MACLCON1 MACLCON2 MAPHSUP\n.        0x{:02X}{:02X} 0x{:02X}{:02X} 0x{:02X}     0x{:02X}     0x{:02X}".format(self.Enc28j60Read(ETXSTH),self.Enc28j60Read(ETXSTL),self.Enc28j60Read(ETXNDH),self.Enc28j60Read(ETXNDL), self.Enc28j60Read(MACLCON1),self.Enc28j60Read(MACLCON2), self.Enc28j60Read(MAPHSUP)))
        print("..........")

    def enc28j60_dump_phyregs(self, msg):
        print("------ {:s} ------".format(msg))
        print("- PHCON1:{:04x}".format(self.Enc28j60PhyRead(PHCON1)))
        print("- PHCON2:{:04x}".format(self.Enc28j60PhyRead(PHCON2)))
        print("- PHSTAT1:{:04x}".format(self.Enc28j60PhyRead(PHSTAT1)))
        print("- PHSTAT2:{:04x}".format(self.Enc28j60PhyRead(PHSTAT2)))
        print("- PHID1:{:04x}".format(self.Enc28j60PhyRead(PHHID1)))
        print("- PHID2:{:04x}".format(self.Enc28j60PhyRead(PHHID2)))
        print("------------")

if __name__ == '__main__':
    import ubinascii
    import usys
    
    enc = ENC28J60()
    enc.Enc28j60Init(b'\x3e\x3f\x3f\x02\x11\x22')
    print("My MAC Adrress:", ubinascii.hexlify(enc.myMacAddr,':'))
    enc.enc28j60_dump_phyregs("PHY Register")
    enc.InitPhy()
    msg = b'\xff\xff\xff\xff\xff\xff' + enc.myMacAddr + b'\x08\x06\x00\x01\x08\x00\x06\x04\x00\x01' + enc.myMacAddr + b'\xc0\xa8\x01\x27\x00\x00\x00\x00\x00\x00\xc0\xa8\x01\x01'
    for i in range(3):
        if enc.Enc28j60IsLinkUp():
            print("Link UP")
        else:
            print("Link DOWN")
    if enc.Enc28j60IsLinkUp():
        pass
    else:
        enc.disableInt()
        enc.Enc28j60deInit()
        usys.exit()

    enc.Enc28j60PacketSend(len(msg), msg)
    packet = bytearray(200)
    n1 = 0
    sec = int(time.time())
    try:
        while True:
            if enc.int28j60:
                enc.int28j60 = False
                len = enc.Enc28j60PacketReceive(200, packet)
                if len > 0:
                    print("Recv:",len,",Data=",ubinascii.hexlify(packet[:len],b' '))
            csec = time.time()
            if (csec - sec) >= 1:
                sec = csec
                n1 += 1
                print(n1 , " Sec ..")
                if n1 >= 15:
                    break
        enc.disableInt()
        enc.Enc28j60deInit()

    except KeyboardInterrupt:
        print('Quit')
        enc.disableInt()
        enc.Enc28j60deInit()


'''
..... Init() Exit .....
. HwRevID: 0x04
. Cntrl: ECON1 ECON2 ESTAT  EIR  EIE
.        0x04  0x80  0x01  0x00  0xC0
. MAC  : MACON1 MACON3 MACON4
.        0x0D   0x32   0x00
. Rx   : ERXST  ERXND  ERXWRPT ERXRDPT ERXFCON EPKTCNT MAMXFL
.        0x0000 0x19FE 0x0000  0x0000  0xB0    0x00    0x05DC
. Tx   : ETXST  ETXND  MACLCON1 MACLCON2 MAPHSUP
.        0x19FF 0x1FFF 0x0F     0x37     0x10
..........
My MAC Adrress: b'3e:3f:3f:02:11:22'
------ PHY Register ------
- PHCON1:0000
- PHCON2:0100
- PHSTAT1:1804
- PHSTAT2:0400
- PHID1:0083
- PHID2:1400
------------
Link UP
Link UP
Link UP
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 02 xx xx xx xx ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
1  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
2  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
3  Sec ..
4  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
Recv: 60 ,Data= b'ff ff ff ff ff ff xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
5  Sec ..
6  Sec ..
Recv: 60 ,Data= b'ff ff ff ff ff ff xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
Recv: 60 ,Data= b'ff ff ff ff ff ff xx xx xx xx ed d5 08 06 00 01 08 00 06 04 00 01 xx xx xx xx ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
7  Sec ..
8  Sec ..
9  Sec ..
10  Sec ..
11  Sec ..
12  Sec ..
13  Sec ..
14  Sec ..
15  Sec ..
'''