import net
import time

'''
 please modify the following two lines. mac and ip have to be unique
 in your local area network. You can not have the same numbers in
 two devices:
'''
mymac = bytes([0x19,0x21,0x68,0x00,0x00,0x29]) 
myip = bytes([192,168,1,39])
mywwwport = 8080;           # listen port for tcp/www

BUFFER_SIZE  = 600

class WWWSERVER:
    def __init__(self):
        self.lanPacket = bytearray(BUFFER_SIZE+1)
        self.colorString = b"background: white;"
        self.colorSet = 0
        self.inet = None            # net instance

    def GetCommand(self, x):
        if b"/ " == x[:2]:
             return 0
        if b"/B" == x[:2]:
            return 1
        if b"/G" == x[:2]:
            return 2
        if b"/R" == x[:2]:
            return 3
        return 0

    def PrintWebpage(self, buf):
        bip = b''
        for n in myip:
            bip += bytes(str(n),'utf-8') + b'.'
        if mywwwport == 80:
            url = b'http://' + bip[:-1]
        else:
            url = b'http://' + bip[:-1] + b':' + bytes(str(mywwwport),'utf-8')
        if self.colorSet == 1:
            colorString = b"background: blue;"
        elif self.colorSet == 2:
            colorString = b"background: lightgreen;"
        elif self.colorSet == 3:
            colorString = b"background: red;"
        else:
            colorString = b"background: white;"
        plen = self.inet.Fill_tcp_data_p(buf, 0, b"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n")
        plen = self.inet.Fill_tcp_data_p(buf, plen, b"<style>body{background: #F0F0F0;}h3{color: blue;}.circle{border-radius: 50%;border: 1px solid black;width: 50px;height: 50px;margin: auto;")
        plen = self.inet.Fill_tcp_data_p(buf,plen,colorString)
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"}</style><center><h1>Welcom to Web Server</h1><div class=\"circle\"></div>")
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"<h3>Click to change Color</h3>")
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"<a href=\"" + url + b"/B\">Set Blue</a><br><br>")
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"<a href=\"" + url + b"/G\">Set Green</a><br><br>")
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"<a href=\"" + url + b"/R\">Set Red</a><br><br>")
        plen = self.inet.Fill_tcp_data_p(buf,plen,b"<a href=\"" + url + b"/\">Clear Color</a>")
        return plen

    def LanTask(self):
        dat_p = 0;
        while True:
            plen = self.inet.enc.Enc28j60PacketReceive(BUFFER_SIZE, self.lanPacket)
            # plen will unequal to zero if there is a valid packet (without crc error)
            if plen == 0:
                return

            # arp is broadcast if unknown but a host may also verify the mac address by sending it to a unicast address.
            if self.inet.Eth_type_is_arp_and_my_ip(self.lanPacket, plen):
                self.inet.Make_arp_answer_from_request(self.lanPacket)
                continue

            # check if the ip packet is for us:
            if self.inet.Eth_type_is_ip_and_my_ip(self.lanPacket, plen) == 0:
                continue
            # ping
            if self.inet.isEchoReply(self.lanPacket):
                self.inet.Make_echo_reply_from_request(self.lanPacket, plen)
                continue

            # tcp port www start, compare only the lower byte
            if self.inet.isTcpPort(self.lanPacket):
                if self.inet.isSYNflag(self.lanPacket):
                    self.inet.Make_tcp_synack_from_syn(self.lanPacket)    # make_tcp_synack_from_syn does already send the syn,ack
                    continue
                if self.inet.isACKflag(self.lanPacket):
                    self.inet.Init_len_info(self.lanPacket)    # init some data structures
                    dat_p = self.inet.Get_tcp_data_pointer()
                    if dat_p == 0:
                        # we can possibly have no data, just ack:
                        if self.inet.isFINflag(self.lanPacket):
                            self.inet.Make_tcp_ack_from_any(self.lanPacket)
                        continue;
                send = False;
                if b"GET " != self.lanPacket[dat_p:dat_p+4]:
                    # head, post and other methods for possible status codes see:
                    # http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
                    plen = self.inet.Fill_tcp_data_p(self.lanPacket, 0, b"HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n<h1>200 OK</h1>")
                    send = true;
                if send == False:
                    #  dat_p+4 : start of incomming payload
                    self.colorSet = self.GetCommand(self.lanPacket[dat_p+4:])
                    plen = self.PrintWebpage(self.lanPacket)
                    send = True

                self.inet.Make_tcp_ack_from_any(self.lanPacket)          # send ack for http get
                self.inet.Make_tcp_ack_with_data(self.lanPacket, plen)   # send data       

    def main(self):
        try:
            while True:
                self.inet = net.NET(mymac,myip,mywwwport)     # cleate instance net arp,udp,tcp and ENC28J60
                self.inet.enc.InitPhy();
                self.inet.Init_ip_arp_udp_tcp()               # init the ethernet/ip layer and enc28j60
                if self.inet.enc.Enc28j60IsLinkUp():
                    break
                self.inet.enc.disableInt()
                print("Link Down")
                time.sleep(1)
            while True:
                if self.inet.enc.int28j60:
                    self.inet.enc.int28j60 = False;
                    self.LanTask()
                # insert your own tasks here !!
        except KeyboardInterrupt:
            print('Quit')
            self.inet.enc.disableInt()
            self.inet.enc.Enc28j60deInit()

if __name__ == '__main__':
    srv = WWWSERVER()
    srv.main()

